#!/usr/bin/python
# Copyright (C) 2013 Linaro Limited
#
# Author: Dave Pigott <dave.pigott@linaro.org>
#
# This file is part of LAVA Dispatcher.
#
# LAVA Lmp is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LAVA Lmp is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along
# with this program; if not, see <http://www.gnu.org/licenses>.

import sys
import getopt
import serial
import logging
from serial import (
    serialutil
)


class LAVALmpException(Exception):
    """
    Base exception class
    """


class LAVALmpDeviceSerial(object):
    def __init__(self, device_path):
        self.device_path = device_path
        try:
            self.port = serial.Serial(device_path, timeout=1)
        except serial.serialutil.SerialException as e:
            logging.error("LMP: Error opening {0:s}: {1:s}".format(self.device_path, e))
            raise LAVALmpException("Could not open serial connection")
        self.START_FRAME = '\x02'
        self.END_FRAME = '\x04'

    def setSerial(self, serialno):
        self.sendFrame('{"schema":"org.linaro.lmp.set-serial","serial":"%s"}' % serialno.zfill(12))

    def sendFrame(self, command):
        logging.debug("LMP: Sending %s" % command)
        payload = self.START_FRAME + command + self.END_FRAME
        print "Sending: %s" % command
        self.port.write(payload)

    def close(self):
        self.port.close()


def usage():
    print ("Usage: %s -d {serial device /dev/ttyACMx} -s {serial number}" % sys.argv[0])


def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], "hs:d:", ["help", "serial", "device"])
    except getopt.error as e:
        print e
        usage()
        sys.exit(2)

    device = "/dev/ttyACM0"
    serialno = None

    for o, v in opts:
        if o in ('-h', '--help'):
            usage()
            sys.exit(0)
        elif o in ('-d', '--device'):
            device = v
        elif o in ('-s', '--serial'):
            serialno = v
        else:
            assert False, "Unknown option %s" % o

    if serialno is None:
        usage()
        assert False, "You must specify a serial number"

    print "Device: %s\nSerial: %012s" % (device, serialno)

    lmp_connection = LAVALmpDeviceSerial(device)
    lmp_connection.setSerial(serialno)
    lmp_connection.close()


if __name__ == '__main__':
    main()